﻿using SimpleVectorMatrixPlayground.VectorMatrix;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SimpleVectorMatrixPlayground.Tests
{
    class AssertionUtil
    {
        private readonly ITestHandler testHandler;

        public AssertionUtil(ITestHandler ts) { testHandler = ts; }

        public void AssertEqual(bool actual, bool expected)
        {
            if (actual == expected)
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("Assert: Actual value = " + actual + ", Expected value = " + expected);
            }
        }

        public void AssertEqual(double actualVal, double expectedVal)
        {
            if (actualVal == expectedVal)
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("AssertEqual: Actual value = " + actualVal + ", Expected value = " + expectedVal);
            }
        }

        public void AssertEqual(double actualVal, double expectedVal, double epsilon)
        {
            if (Math.Abs(actualVal - expectedVal) < epsilon)
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("AssertEqual: Actual value = " + actualVal + ", Expected value = " + expectedVal);
            }
        }

        public void AssertEqual(Vector2 a, Vector2 b)
        {
            if (a.X == b.X && a.Y == b.Y)
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("AssertEqual: Actual values " + Environment.NewLine +
                     a + Environment.NewLine + "Expected values: " + Environment.NewLine + b);
            }
        }

        public void AssertEqual(Vector2 a, Vector2 b, double epsilon)
        {
            if ((Math.Abs(a.X - b.X) < epsilon) && (Math.Abs(a.Y - b.Y) < epsilon))
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("AssertEqual: Actual values " + Environment.NewLine +
                     a + Environment.NewLine + "Expected values: " + Environment.NewLine + b);
            }
        }

        public void AssertException<T>(Action a) where T : Exception
        {
            try
            {
                a.Invoke();
                this.testHandler.TestStepFail("No exception occured.");
            }
            catch (T)
            {
                this.testHandler.TestStepSuccess();
            }
            catch (Exception)
            {
                this.testHandler.TestStepFail("Wrong exception occured.");
            }
        }

        public void AssertEqual(Vector3 a, Vector3 b)
        {
            if (a.X == b.X && a.Y == b.Y && a.Z == b.Z)
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("AssertEqual: Actual values " + Environment.NewLine +
                     a + Environment.NewLine + "Expected values: " + Environment.NewLine + b);
            }
        }

        public void AssertEqual(Vector3 a, Vector3 b, double epsilon)
        {
            if ((Math.Abs(a.X - b.X) < epsilon) && (Math.Abs(a.Y - b.Y) < epsilon) && (Math.Abs(a.Z - b.Z) < epsilon))
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("AssertEqual: Actual values " + Environment.NewLine +
                     a + Environment.NewLine + "Expected values: " + Environment.NewLine + b);
            }
        }


        public void AssertEqual(Matrix2x2 matrix2x2, double m11, double m12, double m21, double m22)
        {
            if (matrix2x2[0, 0] == m11 && matrix2x2[0, 1] == m12 &&
                matrix2x2[1, 0] == m21 && matrix2x2[1, 1] == m22)
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("AssertEqual: Actual values " + Environment.NewLine +
                     matrix2x2 +
                     "Expected values: " + Environment.NewLine +
                    "| " + m11 + " | " + m12 + " |" + Environment.NewLine +
                    "| " + m21 + " | " + m22 + " |" + Environment.NewLine);
            }
        }

        public void AssertEqual(Matrix2x2 m22Exp, Matrix2x2 m22Act)
        {
            if (m22Exp[0, 0] == m22Act[0, 0] && m22Exp[0, 1] == m22Act[0, 1] &&
                m22Exp[1, 0] == m22Act[1, 0] && m22Exp[1, 1] == m22Act[1, 1])
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("AssertEqual: Actual values " + Environment.NewLine +
                     m22Exp + "Expected values: " + Environment.NewLine + m22Act);
            }
        }

        public void AssertEqual(Matrix3x3 matrix3x3, double m11, double m12, double m13,
                                                    double m21, double m22, double m23,
                                                    double m31, double m32, double m33)
        {
            if (matrix3x3[0, 0] == m11 && matrix3x3[0, 1] == m12 && matrix3x3[0, 2] == m13 &&
                matrix3x3[1, 0] == m21 && matrix3x3[1, 1] == m22 && matrix3x3[1, 2] == m23 &&
                matrix3x3[2, 0] == m31 && matrix3x3[2, 1] == m32 && matrix3x3[2, 2] == m33)
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("AssertEqual: Actual values " + Environment.NewLine +
                     matrix3x3 +
                     "Expected values: " + Environment.NewLine +
                    "| " + m11 + " | " + m12 + " |" + m13 + " |" + Environment.NewLine +
                    "| " + m21 + " | " + m22 + " |" + m23 + " |" + Environment.NewLine +
                    "| " + m31 + " | " + m32 + " |" + m33 + " |" + Environment.NewLine);
            }
        }

        public void AssertEqual(Matrix3x3 m33Exp, Matrix3x3 m33Act)
        {
            if (m33Exp[0, 0] == m33Act[0, 0] && m33Exp[0, 1] == m33Act[0, 1] && m33Exp[0, 2] == m33Act[0, 2] &&
                m33Exp[1, 0] == m33Act[1, 0] && m33Exp[1, 1] == m33Act[1, 1] && m33Exp[1, 2] == m33Act[1, 2] &&
                m33Exp[2, 0] == m33Act[2, 0] && m33Exp[2, 1] == m33Act[2, 1] && m33Exp[2, 2] == m33Act[2, 2])
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("AssertEqual: Actual values " + Environment.NewLine +
                     m33Exp + "Expected values: " + Environment.NewLine + m33Act);
            }
        }

        public void AssertEqual(Matrix3x3 m33Exp, Matrix3x3 m33Act, double epsilon)
        {
            if ((Math.Abs(m33Exp[0, 0] - m33Act[0, 0]) < epsilon) &&
                 (Math.Abs(m33Exp[0, 1] - m33Act[0, 1]) < epsilon) &&
                 (Math.Abs(m33Exp[0, 2] - m33Act[0, 2]) < epsilon) &&
                 (Math.Abs(m33Exp[1, 0] - m33Act[1, 0]) < epsilon) &&
                 (Math.Abs(m33Exp[1, 1] - m33Act[1, 1]) < epsilon) &&
                 (Math.Abs(m33Exp[1, 2] - m33Act[1, 2]) < epsilon) &&
                 (Math.Abs(m33Exp[2, 0] - m33Act[2, 0]) < epsilon) &&
                 (Math.Abs(m33Exp[2, 1] - m33Act[2, 1]) < epsilon) &&
                 (Math.Abs(m33Exp[2, 2] - m33Act[2, 2]) < epsilon))
            {
                testHandler.TestStepSuccess();
            }
            else
            {
                testHandler.TestStepFail("AssertEqual: Actual values " + Environment.NewLine +
                     m33Exp + "Expected values: " + Environment.NewLine + m33Act);
            }
        }
    }
}
