﻿using SimpleVectorMatrixPlayground.VectorMatrix;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SimpleVectorMatrixPlayground.Tests
{
    class Mat3x3Tests
    {
        private readonly ITestHandler testHandler;
        private readonly AssertionUtil Assert;

        public Mat3x3Tests(ITestHandler ts)
        {
            this.testHandler = ts;
            Assert = new AssertionUtil(ts);
        }

        public void Run()
        {
            testHandler.TestSuiteStart("Matrix3x3 Tests");

            RunCreationTests();
            RunElementAccessTests();
            RunRowColumnAccessTests();

            RunAdditionSubtractionTests();
            RunMultiplyScalarTests();
            RunMultiplicationTests();
            RunTransposeTests();
            RunNegateTests();
            RunDeterminantTests();
            RunInverseTests();

            RunRotationTestsX();
            RunRotationTestsY();
            RunRotationTestsZ();
            RunScalingTests();
            RunEqualTests();

            testHandler.TestSuiteEnd();
        }

        private void RunCreationTests()
        {
            testHandler.TestGroupStart("Creation");

            Assert.AssertEqual(Matrix3x3.Create(), 0, 0, 0, 0, 0, 0, 0, 0, 0);
            Assert.AssertEqual(Matrix3x3.CreateIdentity(), 1, 0, 0, 0, 1, 0, 0, 0, 1);
            Assert.AssertEqual(Matrix3x3.Create(1, -2, 3.4, 4, -5, 8, 9, 7, 6), 1, -2, 3.4, 4, -5, 8, 9, 7, 6);
            Assert.AssertEqual(Matrix3x3.Create(new double[3, 3] { { 11, 2, -4 }, { 3, 4, 9 }, { 12, 14, 19 } }), 11, 2, -4, 3, 4, 9, 12, 14, 19);

            testHandler.TestGroupEnd();
        }

        private void RunElementAccessTests()
        {
            testHandler.TestGroupStart("Element access");

            Matrix3x3 mat1 = Matrix3x3.Create(-1, 11.1, -2.5, 7.1, 2, 88, -7, -4.5, 0.2);
            Assert.AssertEqual(mat1[0, 0], -1);
            Assert.AssertEqual(mat1[0, 1], 11.1);
            Assert.AssertEqual(mat1[0, 2], -2.5);
            Assert.AssertEqual(mat1[1, 0], 7.1);
            Assert.AssertEqual(mat1[1, 1], 2);
            Assert.AssertEqual(mat1[1, 2], 88);
            Assert.AssertEqual(mat1[2, 0], -7);
            Assert.AssertEqual(mat1[2, 1], -4.5);
            Assert.AssertEqual(mat1[2, 2], 0.2);
            Assert.AssertException<ArgumentOutOfRangeException>(() => mat1[3, 1] = 0);
            Assert.AssertException<ArgumentOutOfRangeException>(() => mat1[1, 3] = 0);

            Matrix3x3 mat2 = Matrix3x3.Create(-1, -25, -4.5, 12, 16, 17, -2, 3, 4.5);
            Assert.AssertEqual(mat2[0], -1);
            Assert.AssertEqual(mat2[1], -25);
            Assert.AssertEqual(mat2[2], -4.5);
            Assert.AssertEqual(mat2[3], 12);
            Assert.AssertEqual(mat2[4], 16);
            Assert.AssertEqual(mat2[5], 17);
            Assert.AssertEqual(mat2[6], -2);
            Assert.AssertEqual(mat2[7], 3);
            Assert.AssertEqual(mat2[8], 4.5);
            Assert.AssertException<ArgumentOutOfRangeException>(() => mat2[9] = 0);

            testHandler.TestGroupEnd();
        }

        private void RunRowColumnAccessTests()
        {
            testHandler.TestGroupStart("Row / Column access");

            Matrix3x3 mat1 = Matrix3x3.Create(-1, 2, 5, -4, 0.25, 13, 3, -23, -11);
            Assert.AssertEqual(mat1.GetRow(0)[0], -1);
            Assert.AssertEqual(mat1.GetRow(0)[1], 2);
            Assert.AssertEqual(mat1.GetRow(0)[2], 5);
            Assert.AssertEqual(mat1.GetRow(1)[0], -4);
            Assert.AssertEqual(mat1.GetRow(1)[1], 0.25);
            Assert.AssertEqual(mat1.GetRow(1)[2], 13);
            Assert.AssertEqual(mat1.GetRow(2)[0], 3);
            Assert.AssertEqual(mat1.GetRow(2)[1], -23);
            Assert.AssertEqual(mat1.GetRow(2)[2], -11);
            Assert.AssertException<ArgumentOutOfRangeException>(() => mat1.GetRow(3));
            Assert.AssertEqual(mat1.GetRowVector(0).X, -1);
            Assert.AssertEqual(mat1.GetRowVector(0).Y, 2);
            Assert.AssertEqual(mat1.GetRowVector(0).Z, 5);
            Assert.AssertEqual(mat1.GetRowVector(1).X, -4);
            Assert.AssertEqual(mat1.GetRowVector(1).Y, 0.25);
            Assert.AssertEqual(mat1.GetRowVector(1).Z, 13);
            Assert.AssertEqual(mat1.GetRowVector(2).X, 3);
            Assert.AssertEqual(mat1.GetRowVector(2).Y, -23);
            Assert.AssertEqual(mat1.GetRowVector(2).Z, -11);
            Assert.AssertException<ArgumentOutOfRangeException>(() => mat1.GetRowVector(3));

            Assert.AssertEqual(mat1.GetColumn(0)[0], -1);
            Assert.AssertEqual(mat1.GetColumn(0)[1], -4);
            Assert.AssertEqual(mat1.GetColumn(0)[2], 3);
            Assert.AssertEqual(mat1.GetColumn(1)[0], 2);
            Assert.AssertEqual(mat1.GetColumn(1)[1], 0.25);
            Assert.AssertEqual(mat1.GetColumn(1)[2], -23);
            Assert.AssertEqual(mat1.GetColumn(2)[0], 5);
            Assert.AssertEqual(mat1.GetColumn(2)[1], 13);
            Assert.AssertEqual(mat1.GetColumn(2)[2], -11);
            Assert.AssertException<ArgumentOutOfRangeException>(() => mat1.GetColumn(3));
            Assert.AssertEqual(mat1.GetColumnVector(0).X, -1);
            Assert.AssertEqual(mat1.GetColumnVector(0).Y, -4);
            Assert.AssertEqual(mat1.GetColumnVector(0).Z, 3);
            Assert.AssertEqual(mat1.GetColumnVector(1).X, 2);
            Assert.AssertEqual(mat1.GetColumnVector(1).Y, 0.25);
            Assert.AssertEqual(mat1.GetColumnVector(1).Z, -23);
            Assert.AssertEqual(mat1.GetColumnVector(2).X, 5);
            Assert.AssertEqual(mat1.GetColumnVector(2).Y, 13);
            Assert.AssertEqual(mat1.GetColumnVector(2).Z, -11);
            Assert.AssertException<ArgumentOutOfRangeException>(() => mat1.GetColumnVector(3));

            testHandler.TestGroupEnd();
        }

        private void RunAdditionSubtractionTests()
        {
            testHandler.TestGroupStart("Addition / Subtraction");

            Assert.AssertEqual(Matrix3x3.CreateIdentity() + Matrix3x3.CreateIdentity(), 2, 0, 0, 0, 2, 0, 0, 0, 2);
            Assert.AssertEqual(Matrix3x3.Create(3, 2, 0, 1, 4, 11, 12, 7, 3) + 
                        Matrix3x3.Create(1, 3, 2, 0, 5, 13, 7, 8, 1), 
                                         4, 5, 2, 1, 9, 24, 19, 15, 4);
            Assert.AssertEqual(Matrix3x3.Create(1, 2, 3, 4, 5, 6, 7, 8, 9) + Matrix3x3.Create(-1, -2, -3, -4, -5, -6, -7, -8, -9), Matrix3x3.Create());

            Assert.AssertEqual(Matrix3x3.Create(2, 0, 0, 0, 2, 0, 0, 0, 2) - Matrix3x3.CreateIdentity(), Matrix3x3.CreateIdentity());
            Assert.AssertEqual(Matrix3x3.Create(4, 5, 2, 1, 5, 8, 2, -3, -4) - 
                        Matrix3x3.Create(3, 2, 0, 1, 6, 3, 5, 4, -9), 
                        Matrix3x3.Create(1, 3, 2, 0, -1, 5, -3, -7, 5));

            testHandler.TestGroupEnd();
        }

        private void RunMultiplyScalarTests()
        {
            testHandler.TestGroupStart("Multiplication (scalar)");

            Assert.AssertEqual(Matrix3x3.CreateIdentity() * 12, 12, 0, 0, 0, 12, 0, 0, 0, 12);
            Assert.AssertEqual(Matrix3x3.Create(3, 2, 0, 1, 5, -6, 7, 4, 12) * -2, -6, -4, 0, -2, -10, 12, -14, -8, -24);

            Assert.AssertEqual(Matrix3x3.CreateIdentity() / 0.5, 2, 0, 0, 0, 2, 0, 0, 0, 2);
            Assert.AssertEqual(Matrix3x3.Create(2, 4, 6, 8, 10, 12, -14, 16, 18) / 2, 1, 2, 3, 4, 5, 6, -7, 8, 9);

            testHandler.TestGroupEnd();
        }

        private void RunMultiplicationTests()
        {
            testHandler.TestGroupStart("Multiplication");

            Assert.AssertEqual(Matrix3x3.CreateIdentity() * Matrix3x3.CreateIdentity(), Matrix3x3.CreateIdentity());
            Assert.AssertEqual(Matrix3x3.Create(1, -5, 3, 0, -2, 6, 7, 2, -4) *
                        Matrix3x3.Create(-8, 6, 1, 7, 0, -3, 2, 4, 5),
                        -37, 18, 31, -2, 24, 36, -50, 26, -19);
            Assert.AssertEqual(Matrix3x3.Create(5, -3, 11, 5, 6, 22, 8, 19, -0) *
                        Matrix3x3.Create(12, 3, -4, 5, 16, 9, 8, -3, 4),
                        133, -66, -3, 266, 45, 122, 191, 328, 139);

            testHandler.TestGroupEnd();
        }

        private void RunTransposeTests()
        {
            testHandler.TestGroupStart("Transpose");

            Assert.AssertEqual(Matrix3x3.CreateIdentity().Transpose(), Matrix3x3.CreateIdentity());
            Assert.AssertEqual(Matrix3x3.Create(1, 2, 3, 4 ,5, 6, 7, 8, 9).Transpose(), Matrix3x3.Create(1, 4, 7, 2, 5, 8, 3, 6, 9));

            testHandler.TestGroupEnd();
        }

        private void RunNegateTests()
        {
            testHandler.TestGroupStart("Negation");

            Assert.AssertEqual(Matrix3x3.CreateIdentity().Negate(), Matrix3x3.Create(-1, 0, 0, 0, -1, 0, 0, 0, -1));
            Assert.AssertEqual(Matrix3x3.Create(1, 2, 3, 4, 5, 6, 7, 10, 11).Negate(), Matrix3x3.Create(-1, -2, -3, -4, -5, -6, -7, -10, -11));

            testHandler.TestGroupEnd();
        }

        private void RunDeterminantTests()
        {
            testHandler.TestGroupStart("Determinant");

            Assert.AssertEqual(Matrix3x3.CreateIdentity().Determinant(), 1);
            Assert.AssertEqual(Matrix3x3.Create().Determinant(), 0);
            Assert.AssertEqual(Matrix3x3.Create(-4, -3, 3, 0, 2, -2, 1, 4, -1).Determinant(), -24);

            testHandler.TestGroupEnd();
        }

        private void RunInverseTests()
        {
            testHandler.TestGroupStart("Inverse");

            Assert.AssertEqual(Matrix3x3.CreateIdentity().Inverse(), Matrix3x3.CreateIdentity());

            Assert.AssertEqual(Matrix3x3.Create(2, -1, 0, 1, 2, -2, 0, -1, 1).Inverse(), 
                        Matrix3x3.Create(0 ,1, 2, -1, 2, 4, -1, 2, 5));
            Assert.AssertEqual(Matrix3x3.Create(2, -1, 0, 1, 2, -2, 0, -1, 1) * Matrix3x3.Create(0, 1, 2, -1, 2, 4, -1, 2, 5), Matrix3x3.CreateIdentity());

            Assert.AssertEqual(Matrix3x3.Create(-4, -3, 3, 0, 2, -2, 1, 4, -1).Inverse(),
                        Matrix3x3.Create(-0.25, -0.375, 0, 
                                         1.0/12.0, -1.0/24.0, 1.0/3.0,
                                         1.0/12.0, -13.0/24.0, 1.0/3.0), 0.00001);
            Assert.AssertEqual(Matrix3x3.Create(-4, -3, 3, 0, 2, -2, 1, 4, -1).Inverse() * Matrix3x3.Create(-4, -3, 3, 0, 2, -2, 1, 4, -1), Matrix3x3.CreateIdentity());
            testHandler.TestGroupEnd();
        }

        private void RunRotationTestsX()
        {
            testHandler.TestGroupStart("RotationX");

            Assert.AssertEqual(Matrix3x3.GetRotationMatrixX(0) * Vector3.Create(2, 2, 2), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixX(Math.PI * 2) * Vector3.Create(2, 2, 2), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixX(Math.PI) * Vector3.Create(2, -2, -2), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixX(Math.PI / 2) * Vector3.Create(5, 1, 0), Vector3.Create(5, 0, 1), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixX(-Math.PI / 2) * Vector3.Create(5, 1, 0), Vector3.Create(5, 0, -1), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixX(Math.PI) * Vector3.Create(5, 1, 0), Vector3.Create(5, -1, 0), 0.00001);

            // multiply a "column" vector to the right side of a matrix is the same as multiplying the same vector
            // treated as "row" vector to the left of the transposed matrix
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixX(Math.PI / 2) * Vector3.Create(5, 1, 0),
                        Vector3.Create(5, 1, 0) * Matrix3x3.GetRotationMatrixX(Math.PI / 2).Transpose(), 0.00001);

            testHandler.TestGroupEnd();
        }

        private void RunRotationTestsY()
        {
            testHandler.TestGroupStart("RotationY");

            Assert.AssertEqual(Matrix3x3.GetRotationMatrixY(0) * Vector3.Create(2, 2, 2), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixY(Math.PI * 2) * Vector3.Create(2, 2, 2), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixY(Math.PI) * Vector3.Create(2, 2, 2), Vector3.Create(-2, 2, -2), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixY(Math.PI / 2) * Vector3.Create(1, 0, 0), Vector3.Create(0, 0, -1), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixY(Math.PI / 2) * Vector3.Create(0, 0, -1), Vector3.Create(-1, 0, 0), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixY(Math.PI) * Vector3.Create(-1, 0, 0), Vector3.Create(1, 0, 0), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixY(Math.PI / 2) * Vector3.Create(5, 5, 1), Vector3.Create(1, 5, -5), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixY(-Math.PI / 2) * Vector3.Create(5, 5, 1), Vector3.Create(-1, 5, 5), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixY(Math.PI) * Vector3.Create(5, 5, 1), Vector3.Create(-5, 5, -1), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixY(-Math.PI / 4) * Vector3.Create(1, 1, 1), Vector3.Create(0, 1, Math.Sqrt(1 + 1)), 0.00001);

            testHandler.TestGroupEnd();
        }

        private void RunRotationTestsZ()
        {
            testHandler.TestGroupStart("RotationZ");

            Assert.AssertEqual(Matrix3x3.GetRotationMatrixZ(0) * Vector3.Create(2, 2, 2), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixZ(Math.PI * 2) * Vector3.Create(2, 2, 2), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixZ(Math.PI) * Vector3.Create(2, 2, 2), Vector3.Create(-2, -2, 2), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixZ(Math.PI / 2) * Vector3.Create(1, 0, 0), Vector3.Create(0, 1, 0), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixZ(Math.PI / 2) * Vector3.Create(0, 1, 0), Vector3.Create(-1, 0, 0), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixZ(Math.PI / 2) * Vector3.Create(-1, 0, 0), Vector3.Create(0, -1, 0), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixZ(Math.PI / 2) * Vector3.Create(5, 1, 5), Vector3.Create(-1, 5, 5), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixZ(-Math.PI / 2) * Vector3.Create(5, 1, 5), Vector3.Create(1, -5, 5), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetRotationMatrixZ(Math.PI) * Vector3.Create(5, 1, 5), Vector3.Create(-5, -1, 5), 0.00001);

            testHandler.TestGroupEnd();
        }

        private void RunScalingTests()
        {
            testHandler.TestGroupStart("Scaling");

            Assert.AssertEqual(Matrix3x3.GetScalingMatrix(3, 3, 3) * Vector3.Create(), Vector3.Create(), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetScalingMatrix(3, 3, 3) * Vector3.Create(1, 1, 1), Vector3.Create(3, 3, 3), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetScalingMatrix(4.5, 1.2, 7.5) * Vector3.Create(4, 2, 3), Vector3.Create(18, 2.4, 22.5), 0.00001);
            Assert.AssertEqual(Matrix3x3.GetScalingMatrix(-3, -1, -2) * Vector3.Create(4, 2, 3), Vector3.Create(-12, -2, -6), 0.00001);

            testHandler.TestGroupEnd();
        }

        private void RunEqualTests()
        {
            testHandler.TestGroupStart("Equal");

            Matrix3x3 m = Matrix3x3.Create(0.1, 1 / 7, -6, -0.9, -8.5, 2.1, -3.5, 2, 89);

            Assert.AssertEqual(m == Matrix3x3.Create(0.1, 1 / 7, -6, -0.9, -8.5, 2.1, -3.5, 2, 89), true);
            Assert.AssertEqual(m != Matrix3x3.Create(0.1, 1 / 7, -6, -0.9, -8.5, 2.1, -3.5, 2, 89), false);
            Assert.AssertEqual(Matrix3x3.CreateIdentity().Equals(Matrix3x3.CreateIdentity()), true);

            testHandler.TestGroupEnd();
        }
    }

}
