﻿
using SimpleVectorMatrixPlayground.VectorMatrix;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SimpleVectorMatrixPlayground.Tests
{
    class Vector3Tests
    {
        private readonly ITestHandler testHandler;
        private readonly AssertionUtil Assert;

        public Vector3Tests(ITestHandler ts) {
            testHandler = ts;
            Assert = new AssertionUtil(ts);
        }

        public void Run()
        {
            testHandler.TestSuiteStart("Vector3Tests");

            RunLengthTests();
            RunBasicArithmeticTests();
            RunNormalizeTests();
            RunDotProductTests();
            RunCrossProductTests();
            RunNegateTests();
            RunInterpolationTests();
            RunAbsOrthogonalColinearTests();
            RunRotatationTests();
            RunScalingTests();
            RunProjectionTests();
            RunSphericalCoordsTests();

            testHandler.TestSuiteEnd();
        }

        private void RunLengthTests()
        {
            testHandler.TestGroupStart("Length / LengthSquared / Distance");

            Assert.AssertEqual(Vector3.Create().GetLength(), 0);
            Assert.AssertEqual(Vector3.Create(1, 0, 0).GetLength(), 1);
            Assert.AssertEqual(Vector3.Create(0, 1, 0).GetLength(), 1);
            Assert.AssertEqual(Vector3.Create(0, 0, 1).GetLength(), 1);
            Assert.AssertEqual(Vector3.Create(0, 3, 4).GetLength(), 5);

            Assert.AssertEqual(Vector3.Create().GetLengthSquared(), 0);
            Assert.AssertEqual(Vector3.Create(1, 1, 1).GetLengthSquared(), 3);
            Assert.AssertEqual(Vector3.Create(3, 4, 6).GetLengthSquared(), 25 + 36);

            Assert.AssertEqual(Vector3.GetDistance(Vector3.Create(), Vector3.Create()), 0);
            Assert.AssertEqual(Vector3.GetDistance(Vector3.Create(1, 0, 0), Vector3.Create()), 1);
            Assert.AssertEqual(Vector3.GetDistance(Vector3.Create(0, 1, 0), Vector3.Create()), 1);
            Assert.AssertEqual(Vector3.GetDistance(Vector3.Create(0, 0, 1), Vector3.Create()), 1);
            Assert.AssertEqual(Vector3.GetDistance(Vector3.Create(), Vector3.Create(1, 0, 0)), 1);
            Assert.AssertEqual(Vector3.GetDistance(Vector3.Create(-1, -2, -3), Vector3.Create(2, 2, 2)), Math.Sqrt(3*3 + 4*4 + 5*5), 0.00001);

            testHandler.TestGroupEnd();
        }

        private void RunBasicArithmeticTests()
        {
            testHandler.TestGroupStart("Basic Arithmetic");

            Assert.AssertEqual(Vector3.Create() + Vector3.Create(), Vector3.Create());
            Assert.AssertEqual(Vector3.Create(1, 0, 1) + Vector3.Create(0, 1, 0), Vector3.Create(1, 1, 1));
            Assert.AssertEqual(Vector3.Create(-1, 0, 7) + Vector3.Create(-2, 8, -1), Vector3.Create(-3, 8, 6));
            Assert.AssertEqual(Vector3.Create(-10.5, 344, -11) + Vector3.Create(10.5, 344, -12), Vector3.Create(0, 688, -23));

            Assert.AssertEqual(Vector3.Create() - Vector3.Create(), Vector3.Create());
            Assert.AssertEqual(Vector3.Create(1, 1, 1) - Vector3.Create(1, 1, 1), Vector3.Create());
            Assert.AssertEqual(Vector3.Create(1, 0, -1) - Vector3.Create(0, 1, -1), Vector3.Create(1, -1, 0));
            Assert.AssertEqual(Vector3.Create(8, -4, 5) - Vector3.Create(-2, -1, 9), Vector3.Create(10, -3, -4));
            Assert.AssertEqual(Vector3.Create(-11, 4, 678) - Vector3.Create(-2, -1, 1.5), Vector3.Create(-9, 5, 676.5));

            Assert.AssertEqual(Vector3.Create(1, -5, 8) * 0, Vector3.Create());
            Assert.AssertEqual(Vector3.Create(1, -5, 7) * 2, Vector3.Create(2, -10, 14));
            Assert.AssertEqual(Vector3.Create(3, -3, -5) * 1.5, Vector3.Create(4.5, -4.5, -7.5));

            Assert.AssertEqual(Vector3.Create() / 1, Vector3.Create());
            Assert.AssertEqual(Vector3.Create(10, 12, -16) / 2, Vector3.Create(5, 6, -8));
            Assert.AssertEqual(Vector3.Create(25, -17.5, 10) / 2.5, Vector3.Create(10, -7, 4));

            testHandler.TestGroupEnd();
        }

        private void RunNormalizeTests()
        {
            testHandler.TestGroupStart("Normalize");

            Assert.AssertEqual(Vector3.Create(1, 0, 0).GetNormalized(), Vector3.Create(1, 0, 0));
            Assert.AssertEqual(Vector3.Create(1, 0, 0).GetNormalized().GetLength(), 1);
            Assert.AssertEqual(Vector3.Create(0, 1, 0).GetNormalized(), Vector3.Create(0, 1, 0));
            Assert.AssertEqual(Vector3.Create(0, 1, 0).GetNormalized().GetLength(), 1);
            Assert.AssertEqual(Vector3.Create(0, 0, 1).GetNormalized(), Vector3.Create(0, 0, 1));
            Assert.AssertEqual(Vector3.Create(0, 0, 1).GetNormalized().GetLength(), 1);

            Assert.AssertEqual(Vector3.Create(3, 4, 5).GetNormalized(), Vector3.Create(3 / Math.Sqrt(50), 4 / Math.Sqrt(50), 5 / Math.Sqrt(50)), 0.0001);
            Assert.AssertEqual(Vector3.Create(3, 4, 5).GetNormalized().GetLength(), 1, 0.00001);
            Assert.AssertEqual(Vector3.Create(-3, 4, -5).GetNormalized(), Vector3.Create(-3 / Math.Sqrt(50), 4 / Math.Sqrt(50), -5 / Math.Sqrt(50)), 0.0001);
            Assert.AssertEqual(Vector3.Create(-3, 4, -5).GetNormalized().GetLength(), 1, 0.00001);

            Assert.AssertEqual(Vector3.Create(12, -5, 0).GetNormalized(), Vector3.Create(12 / 13.0, -5 / 13.0, 0), 0.00001);
            Assert.AssertEqual(Vector3.Create(12, -5, 0).GetNormalized().GetLength(), 1, 0.00001);
            Assert.AssertEqual(Vector3.Create(-5, -12, 0).GetNormalized(), Vector3.Create(-5 / 13.0, -12 / 13.0, 0), 0.00001);
            Assert.AssertEqual(Vector3.Create(-5, -12, 0).GetNormalized().GetLength(), 1, 0.00001);

            Assert.AssertEqual(Double.IsNaN(Vector3.Create().GetNormalized().X), true);
            Assert.AssertEqual(Double.IsNaN(Vector3.Create().GetNormalized().Y), true);
            Assert.AssertEqual(Double.IsNaN(Vector3.Create().GetNormalized().GetLength()), true);

            Assert.AssertEqual(Vector3.Create(1, 1, 1).GetNormalized().GetLength(), 1, 0.00001);
            Assert.AssertEqual(Vector3.Create(-1, -1, -1).GetNormalized().GetLength(), 1, 0.00001);
            Assert.AssertEqual(Vector3.Create(-1, -1,  1).GetNormalized().GetLength(), 1, 0.00001);

            testHandler.TestGroupEnd();
        }

        private void RunDotProductTests()
        {
            testHandler.TestGroupStart("DotProduct");

            // perpendicular
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(), Vector3.Create()), 0);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(1, 0, 0), Vector3.Create(0, 1, 0)), 0);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(1, 0, 0), Vector3.Create(0, 1, 0)), 0);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(1, 0, 0), Vector3.Create(0, 0, 1)), 0);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(0, -1, 0), Vector3.Create(0, 0, -1)), 0);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(-2, 3, 1), Vector3.Create(3, 2, 0)), 0);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(-2, 3, 1), Vector3.Create(-3, -2, 0)), 0);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(0, 4, 0), Vector3.Create(1, 0, 2)), 0);

            // parallel
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(1, 0, 1), Vector3.Create(-1, 0, -1)), -2);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(3, 4, 5), Vector3.Create(3, 4, 5)), 9 + 16 + 25);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(-3, -4, -5), Vector3.Create(3, 4, 5)), -9 + -16 + -25);

            // arbitrary
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(1, 1, 0), Vector3.Create(1, 0, 0)), Math.Sqrt(2) * 1 * Math.Cos(Math.PI / 4), 0.0001);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(1, 0, 1), Vector3.Create(0, 0, 1)), Math.Sqrt(2) * 1 * Math.Cos(Math.PI / 4), 0.0001);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(0, -1, -1), Vector3.Create(0, 1, 0)), -Math.Sqrt(2) * 1 * Math.Cos(Math.PI / 4), 0.0001);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(0, -1, -1), Vector3.Create(0, 0, 1)), -Math.Sqrt(2) * 1 * Math.Cos(Math.PI / 4), 0.0001);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(0, -1, -1), Vector3.Create(0, -1, 0)), Math.Sqrt(2) * 1 * Math.Cos(Math.PI / 4), 0.0001);
            Assert.AssertEqual(Vector3.DotProduct(Vector3.Create(0, -1, -1), Vector3.Create(0, 0, -1)), Math.Sqrt(2) * 1 * Math.Cos(Math.PI / 4), 0.0001);

            testHandler.TestGroupEnd();
        }

        private void RunCrossProductTests()
        {
            testHandler.TestGroupStart("CrossProduct");

            // parallel
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(), Vector3.Create()), Vector3.Create());
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(1, 0, 0), Vector3.Create(1, 0, 0)), Vector3.Create());
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(5, 0, 0), Vector3.Create(3, 0, 0)), Vector3.Create());
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(3, 4, 1), Vector3.Create(-3, -4, -1)), Vector3.Create());

            // b makes counterclockwise turn, so result should point away in a LHS = positive z coordinate
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(1, 0, 0), Vector3.Create(0, 1, 0)), Vector3.Create(0, 0, 1));
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(0, -5, 0), Vector3.Create(4, 0, 0)), Vector3.Create(0, 0, 20));
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(-4, 0, 0), Vector3.Create(0, -3, 0)), Vector3.Create(0, 0, 12));

            // b makes clockwise turn, so result should point towards us in a LHS = negative z coordinate
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(1, 0, 0), Vector3.Create(0, -1, 0)), Vector3.Create(0, 0, -1));
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(0, -5, 0), Vector3.Create(-4, 0, 0)), Vector3.Create(0, 0, -20));
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(-4, 0, 0), Vector3.Create(0, 3, 0)), Vector3.Create(0, 0, -12));

            // left hand rule: thumb in direction of a, index finger in direction c -> third finger gives direction of result vector
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(1, 0, 0), Vector3.Create(0, 0, 2)), Vector3.Create(0, -2, 0));
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(0, 3, 0), Vector3.Create(0, 0, -5)), Vector3.Create(-15, 0, 0));
            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(0, 3, 0), Vector3.Create(-2, 0, 0)), Vector3.Create(0, 0, 6));

            Assert.AssertEqual(Vector3.CrossProduct(Vector3.Create(1, 3, 4), Vector3.Create(2, -5, 8)), Vector3.Create(44, 0, -11));

            testHandler.TestGroupEnd();
        }

        private void RunNegateTests()
        {
            testHandler.TestGroupStart("Negation");

            // negate
            Assert.AssertEqual(Vector3.Create().Negate(), Vector3.Create());
            Assert.AssertEqual(Vector3.Create(1, 2, 3).Negate(), Vector3.Create(-1, -2, -3));
            Assert.AssertEqual(Vector3.Create(1, 0, -3).Negate(), Vector3.Create(-1, 0, 3));
            Assert.AssertEqual(Vector3.Create(-6, -8, -12).Negate(), Vector3.Create(6, 8, 12));
            Assert.AssertEqual(Vector3.Create(1, -3, 0).Negate(), Vector3.Create(-1, 3, 0));
            testHandler.TestGroupEnd();
        }

        private void RunInterpolationTests()
        {
            testHandler.TestGroupStart("Interpolation");

            Assert.AssertEqual(Vector3.Create().GetInterpolation(Vector3.Create(), 0), Vector3.Create());
            Assert.AssertEqual(Vector3.Create().GetInterpolation(Vector3.Create(), -1), Vector3.Create());
            Assert.AssertEqual(Vector3.Create().GetInterpolation(Vector3.Create(), 90), Vector3.Create());

            Assert.AssertEqual(Vector3.Create().GetInterpolation(Vector3.Create(1, 1, 1), 0.5), Vector3.Create(0.5, 0.5, 0.5));
            Assert.AssertEqual(Vector3.Create(1, 0, 2).GetInterpolation(Vector3.Create(-3, 0, 4), 0.5), Vector3.Create(-1, 0, 3));
            Assert.AssertEqual(Vector3.Create(0, -4, -1).GetInterpolation(Vector3.Create(0, 2, -9), 0.25), Vector3.Create(0, -2.5, -3));

            Assert.AssertEqual(Vector3.Create(212, -3232, 311).GetInterpolation(Vector3.Create(1221, 24, 32434), 0), Vector3.Create(212, -3232, 311));
            Assert.AssertEqual(Vector3.Create(-66212, -1.12, -4.5).GetInterpolation(Vector3.Create(433453, -2334.1, -56.7), 1), Vector3.Create(433453, -2334.1, -56.7));

            Assert.AssertEqual(Vector3.Create(1, 1, 1).GetInterpolation(Vector3.Create(2, 2, 2), -1), Vector3.Create());
            Assert.AssertEqual(Vector3.Create(-5, -5, -5).GetInterpolation(Vector3.Create(), 3), Vector3.Create(10, 10, 10));

            testHandler.TestGroupEnd();
        }

        private void RunAbsOrthogonalColinearTests()
        {
            testHandler.TestGroupStart("Absolute / Orthogonal / Colinear");

            Assert.AssertEqual(Vector3.Create().GetAbsolute(), Vector3.Create());
            Assert.AssertEqual(Vector3.Create(-1.2, 6, -3).GetAbsolute(), Vector3.Create(1.2, 6, 3));
            Assert.AssertEqual(Vector3.Create(-2.6, -4, -4.5).GetAbsolute(), Vector3.Create(2.6, 4, 4.5));
            Assert.AssertEqual(Vector3.Create(0.4, -3, 0.8).GetAbsolute(), Vector3.Create(0.4, 3, 0.8));
            Assert.AssertEqual(Vector3.Create(7, 8.7, 12.3).GetAbsolute(), Vector3.Create(7, 8.7, 12.3));

            Assert.AssertEqual(Vector3.Create(1, 0, 0).IsOrthogonal(Vector3.Create(0, 1, 0)), true);
            Assert.AssertEqual(Vector3.Create(1, 0, 0).IsOrthogonal(Vector3.Create(0, -1,0)), true);
            Assert.AssertEqual(Vector3.Create(1, 0, 0).IsOrthogonal(Vector3.Create(-1, 0, 0)), false);
            Assert.AssertEqual(Vector3.Create(1, 0, 0).IsOrthogonal(Vector3.Create(1, 1, 0)), false);
            Assert.AssertEqual(Vector3.Create(0, 0, 1).IsOrthogonal(Vector3.Create(0, -1, 0)), true);

            Assert.AssertEqual(Vector3.Create(1, 0, 0).IsColinear(Vector3.Create(2, 0, 0)), true);
            Assert.AssertEqual(Vector3.Create(1, 0, 0).IsColinear(Vector3.Create(-2, 0, 0)), true);
            Assert.AssertEqual(Vector3.Create(-2, -2, -2).IsColinear(Vector3.Create(5, 5 ,5)), true);
            Assert.AssertEqual(Vector3.Create(1, 0, 0).IsColinear(Vector3.Create(1, 1, 0)), false);

            testHandler.TestGroupEnd();
        }

        private void RunRotatationTests()
        {
            testHandler.TestGroupStart("Rotations");

            // Around x-axis
            Assert.AssertEqual(Vector3.Create(2, 2, 2).RotateX(0), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Vector3.Create(2, 2, 2).RotateX(Math.PI * 2), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Vector3.Create(2, 2, 2).RotateX(Math.PI), Vector3.Create(2, -2, -2), 0.00001);
            Assert.AssertEqual(Vector3.Create(-2, 3, 3).RotateX(Math.PI), Vector3.Create(-2, -3, -3), 0.00001);
            Assert.AssertEqual(Vector3.Create(5, 1, 0).RotateX(Math.PI / 2), Vector3.Create(5, 0, 1), 0.00001);
            Assert.AssertEqual(Vector3.Create(5, 1, 0).RotateX(-Math.PI / 2), Vector3.Create(5, 0, -1), 0.00001);
            Assert.AssertEqual(Vector3.Create(5, 1, 0).RotateX(Math.PI), Vector3.Create(5, -1, 0), 0.00001);

            // Around y-axis
            Assert.AssertEqual(Vector3.Create(2, 2, 2).RotateY(0), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Vector3.Create(2, 2, 2).RotateY(Math.PI * 2), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Vector3.Create(2, 2, 2).RotateY(Math.PI), Vector3.Create(-2, 2, -2), 0.00001);
            Assert.AssertEqual(Vector3.Create(1, 0, 0).RotateY(Math.PI / 2), Vector3.Create(0, 0, -1), 0.00001);
            Assert.AssertEqual(Vector3.Create(0, 0, -1).RotateY(Math.PI / 2), Vector3.Create(-1, 0, 0), 0.00001);
            Assert.AssertEqual(Vector3.Create(-1, 0, 0).RotateY(Math.PI), Vector3.Create(1, 0, 0), 0.00001);
            Assert.AssertEqual(Vector3.Create(5, 5, 1).RotateY(Math.PI / 2), Vector3.Create(1, 5, -5), 0.00001);
            Assert.AssertEqual(Vector3.Create(5, 5, 1).RotateY(-Math.PI / 2), Vector3.Create(-1, 5, 5), 0.00001);
            Assert.AssertEqual(Vector3.Create(5, 5, 1).RotateY(Math.PI), Vector3.Create(-5, 5, -1), 0.00001);
            Assert.AssertEqual(Vector3.Create(1, 1, 1).RotateY(-Math.PI / 4), Vector3.Create(0, 1, Math.Sqrt(1+1)), 0.00001);

            // Around z-axis
            Assert.AssertEqual(Vector3.Create(2, 2, 2).RotateZ(0), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Vector3.Create(2, 2, 2).RotateZ(Math.PI * 2), Vector3.Create(2, 2, 2), 0.00001);
            Assert.AssertEqual(Vector3.Create(2, 2, 2).RotateZ(Math.PI), Vector3.Create(-2, -2, 2), 0.00001);
            Assert.AssertEqual(Vector3.Create(1, 0, 0).RotateZ(Math.PI/2), Vector3.Create(0, 1, 0), 0.00001);
            Assert.AssertEqual(Vector3.Create(0, 1, 0).RotateZ(Math.PI / 2), Vector3.Create(-1, 0, 0), 0.00001);
            Assert.AssertEqual(Vector3.Create(-1, 0, 0).RotateZ(Math.PI / 2), Vector3.Create(0, -1, 0), 0.00001);
            Assert.AssertEqual(Vector3.Create(5, 1, 5).RotateZ(Math.PI / 2), Vector3.Create(-1, 5, 5), 0.00001);
            Assert.AssertEqual(Vector3.Create(5, 1, 5).RotateZ(-Math.PI / 2), Vector3.Create(1, -5, 5), 0.00001);
            Assert.AssertEqual(Vector3.Create(5, 1, 5).RotateZ(Math.PI), Vector3.Create(-5, -1, 5), 0.00001);

            testHandler.TestGroupEnd();
        }

        private void RunScalingTests()
        {
            testHandler.TestGroupStart("Scaling");

            Assert.AssertEqual(Vector3.Create(1, 0, -3).Scale(8, 2, 4), Vector3.Create(8, 0, -12));
            Assert.AssertEqual(Vector3.Create(-2, -3, -0.5).Scale(4, -1.5, -0.5), Vector3.Create(-8, 4.5, 0.25));
            Assert.AssertEqual(Vector3.Create(4, 2, 1).Scale(0.5, 0.25, 0.8), Vector3.Create(2, 0.5, 0.8));

            testHandler.TestGroupEnd();
        }

        private void RunProjectionTests()
        {
            testHandler.TestGroupStart("Projection");

            Assert.AssertEqual(Vector3.GetProjectedVectorLength(Vector3.Create(5, 0, 0), Vector3.Create(3, 2, 0)), 3);
            Assert.AssertEqual(Vector3.GetProjectedVectorLength(Vector3.Create(4, 4, 4), Vector3.Create(4, 4, 4)), Math.Sqrt(16 * 3), 0.00001);
            Assert.AssertEqual(Vector3.GetProjectedVectorLength(Vector3.Create(1, 0, 1), Vector3.Create(1, 1, 1)), Math.Sqrt(2), 0.0001);

            Assert.AssertEqual(Vector3.GetProjectedVectorLength(Vector3.Create(1, 1, 1), Vector3.Create(-1, 1, 0)), 0);
            Assert.AssertEqual(Vector3.GetProjectedVectorLength(Vector3.Create(-1, 0, -1), Vector3.Create(0, 1, 0)), 0);

            Assert.AssertEqual(Vector3.GetProjectedVector(Vector3.Create(5, 0, 0), Vector3.Create(3, 2, 0)), Vector3.Create(3, 0, 0));
            Assert.AssertEqual(Vector3.GetProjectedVector(Vector3.Create(1, 0, 1), Vector3.Create(1, 1, 1)), Vector3.Create(1, 0, 1), 0.00001);

            testHandler.TestGroupEnd();
        }

        private void RunSphericalCoordsTests()
        {
            testHandler.TestGroupStart("Spherical coordinates");

            Assert.AssertEqual(Vector3.GetVectorFromSpherical(1, 0, 0), Vector3.Create(0, 0, 1), 0.00001);
            Assert.AssertEqual(Vector3.GetVectorFromSpherical(1, Math.PI / 2, 0), Vector3.Create(1, 0, 0), 0.00001);
            Assert.AssertEqual(Vector3.GetVectorFromSpherical(1, -Math.PI / 2, 0), Vector3.Create(-1, 0, 0), 0.00001);
            Assert.AssertEqual(Vector3.GetVectorFromSpherical(1, Math.PI, 0), Vector3.Create(0, 0, -1), 0.00001);
            Assert.AssertEqual(Vector3.GetVectorFromSpherical(-5, Math.PI, 0), Vector3.Create(0, 0, 5), 0.00001);
            Assert.AssertEqual(Vector3.GetVectorFromSpherical(1, 0, Math.PI / 2), Vector3.Create(0, 0, 1), 0.00001);
            Assert.AssertEqual(Vector3.GetVectorFromSpherical(1, Math.PI / 2, Math.PI / 2), Vector3.Create(0, 1, 0), 0.00001);
            Assert.AssertEqual(Vector3.GetVectorFromSpherical(1, -Math.PI / 2, Math.PI / 2), Vector3.Create(0, -1, 0), 0.00001);
            Assert.AssertEqual(Vector3.GetVectorFromSpherical(1, Math.PI / 2, -Math.PI / 2), Vector3.Create(0, -1, 0), 0.00001);
            Assert.AssertEqual(Vector3.GetVectorFromSpherical(5, -Math.PI / 2, Math.PI), Vector3.Create(5, 0, 0), 0.00001);


            Assert.AssertEqual(Vector3.Create(0, 0, 1).GetPolarAngle(), 0, 0.00001);
            Assert.AssertEqual(Vector3.Create(1, 0, 0).GetPolarAngle(), Math.PI / 2, 0.00001);
            Assert.AssertEqual(Vector3.Create(1, 0, 1).GetPolarAngle(), Math.PI / 4, 0.00001);
            Assert.AssertEqual(Vector3.Create(-1, 0, 1).GetPolarAngle(), -Math.PI / 4, 0.00001);
            Assert.AssertEqual(Vector3.Create(-1, 0, 0).GetPolarAngle(), -Math.PI / 2, 0.00001);
            Assert.AssertEqual(Vector3.Create(0, 0, -1).GetPolarAngle(), Math.PI, 0.00001);
            Assert.AssertEqual(Vector3.Create(2, 2, 2 ).GetPolarAngle(), 0.95531, 0.00001);

            Assert.AssertEqual(Vector3.Create(0, 0, 1).GetAzimuthalAngle(), 0, 0.00001);
            Assert.AssertEqual(Vector3.Create(0, 1, 0).GetAzimuthalAngle(), 0, 0.00001);
            Assert.AssertEqual(Vector3.Create(1, 0, 0).GetAzimuthalAngle(), Math.PI / 2, 0.00001);
            Assert.AssertEqual(Vector3.Create(-1, 0, 0).GetAzimuthalAngle(), -Math.PI / 2, 0.00001);
            Assert.AssertEqual(Vector3.Create(0, -1, 0).GetAzimuthalAngle(), Math.PI, 0.00001);
            Assert.AssertEqual(Vector3.Create(1, 1, 0).GetAzimuthalAngle(), Math.PI / 4, 0.00001);
            Assert.AssertEqual(Vector3.Create(-1, 1, 0).GetAzimuthalAngle(), -Math.PI / 4, 0.00001);
            Assert.AssertEqual(Vector3.Create(1, -1, 0).GetAzimuthalAngle(), 3 * Math.PI / 4, 0.00001);
            Assert.AssertEqual(Vector3.Create(1, 1, 1).GetAzimuthalAngle(), Math.PI / 4, 0.00001);
            Assert.AssertEqual(Vector3.Create(2, 2, 2).GetAzimuthalAngle(), Math.PI / 4, 0.00001);

            testHandler.TestGroupEnd();
        }
    }
}
