﻿using SimpleVectorMatrixPlayground.Tests;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace SimpleVectorMatrixPlayground
{
    class Testsuite : ITestHandler
    {
        private int currentTestsuiteNumOfFailedTests;
        private string currentTestSuiteName;

        public Dictionary<string, int> failedTestsPerTestsuite;

        public Testsuite()
        {
            failedTestsPerTestsuite = new Dictionary<string, int>();
        }

        public void Run()
        {
            Vector2Tests v2Tests = new Vector2Tests(this);
            v2Tests.Run();

            Vector3Tests v3Tests = new Vector3Tests(this);
            v3Tests.Run();

            Mat2x2Tests mat22Tests = new Mat2x2Tests(this);
            mat22Tests.Run();

            Mat3x3Tests mat33Tests = new Mat3x3Tests(this);
            mat33Tests.Run();

            Console.WriteLine("=========");
            Console.WriteLine("Summary:");
            foreach (string key in failedTestsPerTestsuite.Keys)
            {
                Console.WriteLine(" - " + key + ": " + (failedTestsPerTestsuite[key] == 0 ? "PASSED" : failedTestsPerTestsuite[key].ToString()));
            }

            int totalNumOfFailedTests = failedTestsPerTestsuite.Values.Sum();
            Console.WriteLine("\nTotal number of failed testcases: " + totalNumOfFailedTests);
        }

        public void TestSuiteStart(string name)
        {
            Console.WriteLine("----------------------------");
            Console.WriteLine("Running test suite: " + name);
            currentTestSuiteName = name;
            currentTestsuiteNumOfFailedTests = 0;
        }

        public void TestSuiteEnd()
        {
            Console.WriteLine();
            failedTestsPerTestsuite.Add(currentTestSuiteName, currentTestsuiteNumOfFailedTests);

            if (currentTestsuiteNumOfFailedTests == 0)
            {
                Console.WriteLine("Test: PASSED");
            }
            else
            {
                Console.WriteLine("Test: FAILED - " + currentTestsuiteNumOfFailedTests + " test steps :-(");
            }
            Console.WriteLine("----------------------------");
            Console.WriteLine();
        }

        public void TestGroupStart(string name)
        {
            Console.Write(name + ": ");
        }

        public void TestGroupEnd()
        {
            Console.WriteLine();
        }

        public void TestStepSuccess()
        {
            Console.Write('x');
        }

        public void TestStepFail(string message)
        {
            Console.WriteLine();
            Console.WriteLine("F: " + message);
            currentTestsuiteNumOfFailedTests++;
        }
    }
}
